using System;
using System.Configuration;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Timers;
using System.Threading;
using System.Diagnostics;
using System.ServiceProcess;
using System.Runtime;
using System.Web;
using System.Net;
using System.IO;
using System.Text;
using HumanityServices;

namespace HumanityServices
{

	/// <summary>
	/// Class: WebRefresher 
	/// 2005,2006 Advance Guard Technologies - All Rights reserved
	/// Created: 2005.05.06 by ken norwick
    /// Last Updated: 2006.06.10 - Added Daily Tasks Processing
	/// This class encapsulates all of the background processing service tasks
	/// for the Humanity Enterprise Portal such as the KeepReadyService, etc.
	/// The default timer event fires every 5 minutes by default, but this behaviour
	/// can be changed by modifying the setting "KeepAliveInterval" in the configuration
	/// file for the service.
	/// </summary>
	public class WebRefresher : System.ServiceProcess.ServiceBase
	{

		/// <summary>
		/// Property: KeepAliveWebs
		/// An ArrayList of KeepAliveWeb objects containing all of the enabled webs
		/// to be processed by this background service component.
		/// </summary>
		protected ArrayList KeepAliveWebs;

		private System.Timers.Timer HumanityTimer;
		
		private System.ComponentModel.IContainer components = null;

		/// <summary>
		/// Property: PrimaryKeepAliveURL
		/// This string contains the primary web site's URL. This site is always
		/// processed by the service.
		/// </summary>
		protected String PrimaryKeepAliveURL = "";

		/// <summary>
		/// Property: PrimaryPhysicalPath
		/// String containing the physical path to the primary web application root
		/// </summary>
		protected String PrimaryPhysicalPath = "";

		/// <summary>
		/// Property: KeepAliveURL (String)
		/// This string contains the root URL of the .NET application that the
		/// service will attempt to access every n minutes.
		/// </summary>
		protected String KeepAliveURL = "";

		/// <summary>
		/// Property: KeepAlivePhysicalPath (String)
		/// This string contains the physical path on the server where the app
		/// can be located. This should point to the app's root folder on the server,
		/// and is the starting point for the recursive expedition by the service
		/// through the application's sub-directories in search of .NET pages or
		/// user controls that must be accessed.  Accessing these objects causes the
		/// .NET runtime to re-compile the object. Web site visitors will not be subjected
		/// to any delays when a portal page is accessed by them. The site will "rip"!!!
		/// </summary>
		protected String KeepAlivePhysicalPath = "";

		/// <summary>
		/// Property: WebPagePrimary
		/// </summary>
		protected String WebPagePrimary = "Default.aspx";
		/// <summary>
		/// Property: WebPageSecondary
		/// </summary>
		protected String WebPageSecondary = "MasterPage.aspx";

        protected String WebPageDailyTasks = "TTLExpiry.aspx";

		/// <summary>
		/// Property: PauseInMilliseconds 
		/// Indicates the number of milliseconds that this thread should be paused to
		/// allow main web requests to be serviced.
		/// </summary>
		protected int PauseInMilliseconds = 0;

		/// <summary>
		/// Property: iMaximumRefreshedWebs
		/// Contains the maximum number of web sites that should be refreshed by this
		/// Windows Service application. This value is read from the configuration
		/// file whenever this service starts/restarts
		/// </summary>
		protected int iMaximumRefreshedWebs = 25;

		private System.Timers.Timer OnStartTimer;


		/// <summary>
		/// Property: KeepAliveLogActivity (String)
		/// This string contains a yes/no setting that indicates whether or not the service
		/// should write a summary to the system application log each time an event occurs
		/// in the service.
		/// </summary>
		protected bool KeepAliveLogActivity = false;

		/// <summary>
		/// Property: TasksPending
		/// This indicates whether or not to start a new KeepAlive cycle on the
		/// timer event. We just wait another timer interval if we are not done
		/// with the current task set.
		/// </summary>
		protected bool TasksPending = false;

		/// <summary>
		/// Constructor: WebRefresher()
		/// 2005,2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2004.05.06 by ken norwick
		/// This constructor is the entry point for the HumanityWebRefresher.WebRefresher
		/// service that provides background support processing for Humanity.
		/// </summary>
		public WebRefresher()
		{
			// This call is required by the Windows.Forms Component Designer.
			InitializeComponent();
		}

		// The main entry point for the process
		static void Main()
		{
			System.ServiceProcess.ServiceBase[] ServicesToRun;
	
			// More than one user Service may run within the same process. To add
			// another service to this process, change the following line to
			// create a second service object. For example,
			//
			//   ServicesToRun = new System.ServiceProcess.ServiceBase[] {new Service1(), new MySecondUserService()};
			//
			ServicesToRun = new System.ServiceProcess.ServiceBase[] { new WebRefresher() };

			System.ServiceProcess.ServiceBase.Run(ServicesToRun);
		}

		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.HumanityTimer = new System.Timers.Timer();
			this.OnStartTimer = new System.Timers.Timer();
			((System.ComponentModel.ISupportInitialize)(this.HumanityTimer)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.OnStartTimer)).BeginInit();
			// 
			// OnStartTimer
			// 
			this.OnStartTimer.AutoReset = false;
			this.OnStartTimer.Interval = 10000;
			// 
			// KeepAlive
			// 
			this.CanPauseAndContinue = true;
			this.ServiceName = "WebRefresher";
			((System.ComponentModel.ISupportInitialize)(this.HumanityTimer)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.OnStartTimer)).EndInit();

		}

	
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		/// <summary>
		/// EventHandler: OnStart()
		/// 2005,2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2004.05.06 by ken norwick
		/// This method will handle the OnStart event for the WebRefresh service. 
		/// It begins by reading the application configuration
		/// file to retrieve the current service settings for timer interval, etc.
		/// It then executes its task list for the first time to prepare the specified
		/// .NET directory for immediate use. It does this by recursively parsing the 
		/// directory structure under the application root looking for all .NET file
		/// types such as .aspx, .ascx. When it encounters such a file it makes an
		/// asynchronous call to the object which causes the .NET CLR to compile the
		/// object if required. As well, this action loads the files into the Humanity 
		/// Portal object cache and the portal can run at its top speed without pausing
		/// to compile its pages for users. This background tasks takes care of this 
		/// activity automatically, and in the background.
		/// Once the pages have been initialized for the first time, this method enables
		/// the system timer and sets its interval to the configured value. It then creates
		/// an event handler to process the timer interval elapsed event. The process then
		/// repeats every N seconds while the service is running.
		/// </summary>
		protected override void OnStart(string[] args)
		{

			try
			{

				OnStartTimer.AutoReset = false;
				OnStartTimer.Enabled = true;
				OnStartTimer.Interval = 10000;
				
				OnStartTimer.Elapsed += 
					new System.Timers.ElapsedEventHandler(OnStartTimer_Elapsed);

				OnStartTimer.Start();

			}
			catch(Exception ex)
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "Error (OnStart event): " + ex.Message);
			}

		}
 
		private void OnStartTimer_Elapsed(object sender, 
			System.Timers.ElapsedEventArgs e)
		{
			try
			{
				OnStartTimer.Enabled = false;
	
				EventLog.WriteEntry("*HumanityTHUNDER", "OnStartTimer_Elapsed event raised. Checking service status.");
				ServiceController service = new ServiceController("WebRefresher");
				if(service != null)
				{
					if ( service.Status != ServiceControllerStatus.Running )
					{
						EventLog.WriteEntry("*HumanityTHUNDER", "OnStartTimer_Elapsed event. Waiting on Service to start.");
						service.Start();

						while(service.Status != ServiceControllerStatus.Running)
						{
							Thread.Sleep(0);
						}
					}

					EventLog.WriteEntry("*HumanityTHUNDER", "Continue Processing event. The Service is running. Reading configuration files now.");
					GetConfigurationSettings();
					GetConfiguredWebs();

					if(KeepAliveLogActivity == true)
						EventLog.WriteEntry("*HumanityTHUNDER", "OnStartTimer_Elapsed event. Configuration files have been read.");

					// TODO: Add code here to start your service.
					HumanityTimer.AutoReset = true;
					HumanityTimer.Enabled = true;
					HumanityTimer.Interval = SetTimerInterval();
					HumanityTimer.Start();
				
					HumanityTimer.Elapsed += 
						new System.Timers.ElapsedEventHandler(HumanityTimer_Elapsed);

					if(KeepAliveLogActivity == true)
						EventLog.WriteEntry("*HumanityTHUNDER", "OnStartTimer_Elapsed event. Main system timer has started. Starting to process task list.");

					
					ProcessTaskList();


				}

				

				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("*HumanityTHUNDER", "Task Initialization completed.");
				
			}
			catch(Exception ex)
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("***HumanityTHUNDER", "Error (OnStartTimer_Elapsed): " + ex.Message);
			}
		}


		/// <summary>
		/// EventHandler: HumanityTimer_Elapsed
		/// 2005 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.06 by ken norwick
		/// This function handles the timer interval elapsed event for the
		/// service. It simply processes the service's task list and recursively
		/// visits all .NET type objects under the specified URL.
		/// </summary>
		/// <param name="sender">object</param>
		/// <param name="e">System.Timers.ElapsedEventArgs</param>
		private void HumanityTimer_Elapsed(object sender, 
			System.Timers.ElapsedEventArgs e)
		{
			if(KeepAliveLogActivity == true)
				EventLog.WriteEntry("*****HumanityTHUNDER", "HumanityTimer_Elapsed: Checking current task completion.");

			try
			{
				if(TasksPending == false)
				{
					ProcessTaskList();
				}
				else
				{
					if(KeepAliveLogActivity == true)
						EventLog.WriteEntry("HumanityTHUNDER", "HumanityTimer_Elapsed: Current tasks have not completed, cycling timer again.");

				}
			}
			catch(Exception ex)
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("**HumanityTHUNDER", "Error (HumanityTimer_Elapsed): " + ex.Message);
			}
			
		}


		/// <summary>
		/// Method: DoWebRefresherServiceTasks()
		/// 2005,2006 Advance Guard Technologies - All Rights reserved
		/// This function makes an AsynchWebRequest to the primary, the secondary, and the daily task
		/// web pages specified for the current KeepAliveURL. Making these requests triggers the page and all of its related
		/// objects to be compiled and cached; thereby "refreshing" the web site's main pages and business objects.
		/// </summary>
		/// <param name="KeepAliveURL"></param>
		/// <param name="PrimaryWebPage"></param>
		/// <param name="SecondaryWebPage"></param>
		private void DoWebRefresherServiceTasks(String KeepAliveURL, String PrimaryWebPage, String SecondaryWebPage, String DailyTasksWebPage)
		{

			String URLToLaunch = "";
			try
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("* * *HumanityTHUNDER", "KeepReadyServiceTasks - Processing web app at URL: " + KeepAliveURL + "...");

				if((KeepAliveURL != null) && (KeepAliveURL.Trim() != ""))
				{
					if(PrimaryWebPage != null && PrimaryWebPage.ToLower().Trim() != "none" && PrimaryWebPage.Trim() != "")
					{
						URLToLaunch = KeepAliveURL + "/" + PrimaryWebPage;
						if(KeepAliveLogActivity == true)
							EventLog.WriteEntry("HumanityTHUNDER", "Attempting to access object: " + URLToLaunch);	
						//HttpWebRequest.Create(URLToLaunch).BeginGetResponse(null, null);

						AsynchWebRequest _webRequest1 = new AsynchWebRequest(URLToLaunch, KeepAliveLogActivity);

						if(KeepAliveLogActivity == true)
							EventLog.WriteEntry("HumanityTHUNDER", "DoWebRefresherServiceTasks()--> Asychronous request placed to object at URL: " + URLToLaunch);	
					}

					Thread.Sleep(0);	//yield to other processes...
					if(PauseInMilliseconds > 0)
					{
						Thread.Sleep(PauseInMilliseconds);
					}

					if(SecondaryWebPage != null && SecondaryWebPage.ToLower().Trim() != "none" && SecondaryWebPage.Trim() != "")
					{
						URLToLaunch = KeepAliveURL + "/" + SecondaryWebPage;
						if(KeepAliveLogActivity == true)
							EventLog.WriteEntry("HumanityTHUNDER", "Attempting to access object: " + URLToLaunch);	
						//HttpWebRequest.Create(URLToLaunch).BeginGetResponse(null, null);

						AsynchWebRequest _webRequest2 = new AsynchWebRequest(URLToLaunch, KeepAliveLogActivity);

						if(KeepAliveLogActivity == true)
							EventLog.WriteEntry("HumanityTHUNDER", "DoWebRefresherServiceTasks()--> Asychronous request placed to object at URL: " + URLToLaunch);	
					}

					Thread.Sleep(0);	//yield to other processes...
					if(PauseInMilliseconds > 0)
					{
						Thread.Sleep(PauseInMilliseconds);
					}

                    DateTime _current = DateTime.Now;

                    int _minute = _current.Minute;

                    //The application tasks component should only fire maybe once or twice during each hour
                    if (_minute >= 1 && _minute <= 10)
                    {
                        if (DailyTasksWebPage != null && DailyTasksWebPage.ToLower().Trim() != "none" && DailyTasksWebPage.Trim() != "")
                        {
                            URLToLaunch = KeepAliveURL + "/" + DailyTasksWebPage;
                            EventLog.WriteEntry("HumanityTHUNDER", "Attempting to process Application Tasks Page: " + URLToLaunch);
                            AsynchWebRequest _webRequest2 = new AsynchWebRequest(URLToLaunch, KeepAliveLogActivity);

                            EventLog.WriteEntry("HumanityTHUNDER", "DoWebRefresherServiceTasks()--> Asychronous request placed to object at URL: " + URLToLaunch);
                        }
                    }
                    else
                    {
                        if (KeepAliveLogActivity == true)
                            EventLog.WriteEntry("HumanityTHUNDER", "DoWebRefresherServiceTasks()--> Application Tasks process is marking time. Current MINUTE: " + _minute.ToString("##"));	
                    }




                    Thread.Sleep(0);	//yield to other processes...
                    if (PauseInMilliseconds > 0)
                    {
                        Thread.Sleep(PauseInMilliseconds);
                    }

				}
			}
			catch
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("*Error HumanityTHUNDER", "Unable to process Humanity-specific special objects.");	
			}
		}


		/// <summary>
		/// Method: ProcessTaskList()
		/// 2004,2005,2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2004.04.03 by ken norwick
		/// This function will do the main work of the service. It will use settings
		/// from the configuration file to determine which Humanity webs to process.
		/// </summary>
		protected void ProcessTaskList()
		{
			try
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "* * * Timer Event. TasksPending flag has been set to TRUE. Starting web application refresh cycle now... ");

				TasksPending = true;

				//Process Primary web target first
				KeepAliveURL = PrimaryKeepAliveURL;
				KeepAlivePhysicalPath = PrimaryPhysicalPath;

				if((PrimaryPhysicalPath == null) || (PrimaryPhysicalPath.Trim() == ""))
				{
					GetConfigurationSettings();
					GetConfiguredWebs();
					KeepAliveURL = PrimaryKeepAliveURL;
					KeepAlivePhysicalPath = PrimaryPhysicalPath;
				}

				//Note: we will only allow this service to process valid (ie existing) web sites and pages. The following marker file must
				//exist in the specified web application root, or tough luck!
				
				if((PrimaryPhysicalPath != null) && (PrimaryPhysicalPath.Trim() != ""))
				{
					FileInfo _HumanityMustExistFile = new FileInfo(PrimaryPhysicalPath + WebPagePrimary);

					if(_HumanityMustExistFile.Exists == true)
					{
                        DoWebRefresherServiceTasks(PrimaryKeepAliveURL, WebPagePrimary, WebPageSecondary, WebPageDailyTasks);
					}
					else
					{
						//write exception regardless of logging state
						EventLog.WriteEntry("Exception***HumanityTHUNDER", "NOT_VALID_WEB Exception (ProcessTaskList): " + PrimaryPhysicalPath + WebPagePrimary  + " is NOT a valid primary web page, and cannot be serviced by this component. Proceeding to process secondary (if any) webs.");
					}
				}

				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("* * *HumanityTHUNDER", "ProcessTaskList - Processing secondary (if any) webs now...");

				// REMOVE comments below on Standard Edition

				try
				{
					if(KeepAliveWebs.Count > 0)
					{
						TasksPending = true;
						if(KeepAliveLogActivity == true)
							EventLog.WriteEntry("HumanityTHUNDER", "KeepAliveWebs - Found this number of webs: " + KeepAliveWebs.Count.ToString() + " to process...");

						foreach( KeepAliveWeb _web in KeepAliveWebs)
						{
							if(KeepAliveLogActivity == true)
								EventLog.WriteEntry("* * *", "KeepAliveWebs - Found this web: " + _web.KeepAliveWebURL + " to process...");
							KeepAliveURL = _web.KeepAliveWebURL;
							KeepAlivePhysicalPath = _web.KeepAliveWebPhysicalRoot;

							DoWebRefresherServiceTasks(_web.KeepAliveWebURL,_web.KeepAliveWebPagePrimary,_web.KeepAliveWebPageSecondary, _web.KeepAliveWebPageDailyTasks);

							Thread.Sleep(0);	//yield to other processes...
							//interject a "pause" between HttpWebRequest to prevent the web server from
							//becoming too busy to service "real" user requests...
							if(PauseInMilliseconds > 0)
							{
								Thread.Sleep(PauseInMilliseconds);
							}

							//Interject the Primary KeepAlive target web between
							//each "secondary" webs to process...
							//KeepReadyServiceTasks(PrimaryKeepAliveURL);
							//Thread.Sleep(0);	//yield to other processes...
							//if(PauseInMilliseconds > 0)
							//{
							//	Thread.Sleep(PauseInMilliseconds);
							//}
						}
					}
					else
					{
						if(KeepAliveLogActivity == true)
							EventLog.WriteEntry("* * *HumanityTHUNDER", "ProcessTaskList - No secondary webs were found...");

					}

				}
				catch
				{

				}

				TasksPending = false;
				
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "* * * Timer Event. TasksPending flag has been set to FALSE. Ending KeepAlive cycle now... ");


			}
			catch(Exception ex)
			{
				//make sure that the timer can get reset even if can only partially compile a web...
				TasksPending = false;
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("ERROR HumanityTHUNDER", "Error (ProcessTaskList): " + ex.Message);
			}
		}

		/// <summary>
		/// Method: GetConfiguredWebs()
		/// 2005, 2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.06 by ken norwick
		/// This function will read all Humanity Webs configured for the KeepAlive
		/// service and will store them in an ArrayList of KeepAliveWeb objects for
		/// use by the service task list processor.
		/// </summary>
		protected void GetConfiguredWebs()
		{
			try
			{
				if(KeepAliveWebs != null)
				{
					KeepAliveWebs.Clear();
				}
				else
				{
					KeepAliveWebs = new ArrayList();
				}

				KeepAliveWeb _KeepAliveWeb = new KeepAliveWeb();

				for(int i=1;i < iMaximumRefreshedWebs + 1;i++)
				{
					if(KeepAliveLogActivity == true)
						EventLog.WriteEntry("HumanityTHUNDER", "Status (GetConfiguredWebs): Attempting to get secondary keep ready web app #" + i.ToString());
					_KeepAliveWeb = GetConfiguredWeb(i);
					if(_KeepAliveWeb != null)
					{
						KeepAliveWebs.Add(_KeepAliveWeb);	
						if(KeepAliveLogActivity == true)
							EventLog.WriteEntry("* * *HumanityTHUNDER", "Status (GetConfiguredWebs): Secondary keep ready web # " + i.ToString() + " added to current array of managed webs.");					}
				}


			}
			catch(Exception ex)
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "Error (GetConfiguredWebs): " + ex.Message);
			}

		}//end of method

		/// <summary>
		/// Method: GetConfiguredWeb()
		/// 2005,2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.14 by ken norwick
		/// This private function reads the configuration file for the secondary web at INDEX
		/// </summary>
		/// <param name="WebIndex">int - the index of the configured web site within the configuration file</param>
		/// <returns>An initialized KeepAliveWeb object or a null object if an invalid web was specified</returns>
		private KeepAliveWeb GetConfiguredWeb(int WebIndex)
		{

			StringBuilder sbKeepAliveURL = new StringBuilder();
			StringBuilder sbKeepAliveWebPhysicalRoot = new StringBuilder();
			StringBuilder sbKeepAliveWebPagePrimary = new StringBuilder();
			StringBuilder sbKeepAliveWebPageSecondary = new StringBuilder();
            StringBuilder sbKeepAliveWebPageDailyTasks = new StringBuilder();

            KeepAliveWeb _web = new KeepAliveWeb();
			try
			{
				_web.KeepAliveWebID = WebIndex.ToString();

				sbKeepAliveURL.Append("KeepAliveURL" + WebIndex.ToString());
                //Sample 2.0 Replacement method: ConfigurationManager.AppSettings["ReleaseDate"]
                _web.KeepAliveWebURL = ConfigurationManager.AppSettings[sbKeepAliveURL.ToString()];
				if((_web.KeepAliveWebURL == null) || (_web.KeepAliveWebURL.ToLower().IndexOf("not specified") > -1))
				{
					_web.KeepAliveWebURL ="http://localhost/HumanityPortal" + WebIndex.ToString();
				}
			
				if(_web.KeepAliveWebURL.Trim().ToLower() != "not specified")
				{
					sbKeepAliveWebPhysicalRoot.Append("KeepAlivePhysicalRoot" + WebIndex.ToString());
                    _web.KeepAliveWebPhysicalRoot = ConfigurationManager.AppSettings[sbKeepAliveWebPhysicalRoot.ToString()];
					if(_web.KeepAliveWebPhysicalRoot == null)
					{
						_web.KeepAliveWebPhysicalRoot ="c:\\Inetpub\\wwwroot\\HumanityPortal" + WebIndex.ToString() + "\\";
					}
					else
					{
						if(_web.KeepAliveWebPhysicalRoot.EndsWith("\\") == false)
						{
							_web.KeepAliveWebPhysicalRoot = _web.KeepAliveWebPhysicalRoot + "\\";
						}
					}


					//get the primary and secondary web pages for this application...
					sbKeepAliveWebPagePrimary.Append("KeepAliveWebPagePrimary" + WebIndex.ToString());
                    _web.KeepAliveWebPagePrimary = ConfigurationManager.AppSettings[sbKeepAliveWebPagePrimary.ToString()];
					if((_web.KeepAliveWebPagePrimary == null) || (_web.KeepAliveWebPagePrimary.ToLower().IndexOf("none") > -1))
					{
						_web.KeepAliveWebPagePrimary ="Default.aspx";
					}


					sbKeepAliveWebPageSecondary.Append("KeepAliveWebPageSecondary" + WebIndex.ToString());
                    _web.KeepAliveWebPageSecondary = ConfigurationManager.AppSettings[sbKeepAliveWebPageSecondary.ToString()];
					if((_web.KeepAliveWebPageSecondary == null) || (_web.KeepAliveWebPageSecondary.ToLower().IndexOf("none") > -1))
					{
						_web.KeepAliveWebPageSecondary ="Default.aspx";
					}


                    sbKeepAliveWebPageDailyTasks.Append("DailyTasksWebPage" + WebIndex.ToString());
                    _web.KeepAliveWebPageDailyTasks = ConfigurationManager.AppSettings[sbKeepAliveWebPageDailyTasks.ToString()];
                    if ((_web.KeepAliveWebPageDailyTasks == null) || (_web.KeepAliveWebPageDailyTasks.ToLower().IndexOf("none") > -1))
                    {
                        _web.KeepAliveWebPageDailyTasks = "TTLExpiry.aspx";
                    }

					//Note: we will only allow this service to process valid web sites and existing web pages. The following marker file must
					//exist in the specified web application root, or tough luck!
					FileInfo _HumanityMustExistFile = new FileInfo(_web.KeepAliveWebPhysicalRoot + _web.KeepAliveWebPagePrimary);

					if((_web.KeepAliveWebURL.Trim() == "") || (_web.KeepAliveWebPhysicalRoot.Trim() == ""))
					{
						return null;
					}
					else
					{
						if(_HumanityMustExistFile.Exists == true)
						{
							return _web;
						}
						else
						{
							if(KeepAliveLogActivity == true)
								EventLog.WriteEntry("Exception***HumanityTHUNDER", "NOT_VALID_WEB Exception (GetConfiguredWeb): " + _web.KeepAliveWebID  + " is NOT a valid (existing) web, and cannot be serviced by this component...");
							return null;
						}
					}
				}
				else
				{
					//not supported feature found...
					return null;
				}
			
			}
			catch(Exception ex)
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "Error (GetConfiguredWeb1): " + _web.KeepAliveWebID  + " " + ex.Message);
				return null;
			}
		}//end of method

		/// <summary>
		/// Method: GetConfigurationSettings()
		/// 2005 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.06 by ken norwick
		/// This function reads and stores the current service settings from
		/// the configuration file. If any settings are missing or incorrect,
		/// then default (safe) values will be used for processing.
		/// </summary>
		protected void GetConfigurationSettings()
		{
			try
			{
                PrimaryKeepAliveURL = ConfigurationManager.AppSettings["KeepAliveURL"];
				if(PrimaryKeepAliveURL == null)
					PrimaryKeepAliveURL ="http://localhost/YourMainWebSite";
				KeepAliveURL = PrimaryKeepAliveURL;

                PrimaryPhysicalPath = ConfigurationManager.AppSettings["KeepAlivePhysicalRoot"];
				if(PrimaryPhysicalPath == null)
				{
					PrimaryPhysicalPath ="c:\\Inetpub\\wwwroot\\YourMainWebSite\\";
				}
				else
				{
					if(PrimaryPhysicalPath.EndsWith("\\") == false)
					{
						PrimaryPhysicalPath = PrimaryPhysicalPath + "\\";
					}
				}
				KeepAlivePhysicalPath = PrimaryPhysicalPath;


                WebPagePrimary = ConfigurationManager.AppSettings["KeepAliveWebPagePrimary"];
				if(WebPagePrimary == null)
					WebPagePrimary ="Default.aspx";
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "Web Page (Primary): " + WebPagePrimary);


                WebPageSecondary = ConfigurationManager.AppSettings["KeepAliveWebPageSecondary"];
				if(WebPageSecondary == null)
					WebPageSecondary ="Default.aspx";
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "Web Page (Secondary): " + WebPageSecondary);

                WebPageDailyTasks = ConfigurationManager.AppSettings["DailyTasksWebPage"];
                if (WebPageDailyTasks == null)
                    WebPageDailyTasks = "TTLExpiry.aspx";
                if (KeepAliveLogActivity == true)
                    EventLog.WriteEntry("HumanityTHUNDER", "Web Page (Daily Tasks): " + WebPageDailyTasks);


				//KeepAlivePauseMs - Duration (if any) of pause in milliseconds
				//0= no pause, with a maximum of 10000 (10 seconds) allowed
                String _KeepAlivePauseMs = ConfigurationManager.AppSettings["KeepAlivePauseMs"];
				PauseInMilliseconds = Int32.Parse(_KeepAlivePauseMs);
				if((PauseInMilliseconds < 1000) || (PauseInMilliseconds > 10000))
				{
					PauseInMilliseconds = 0;
				}
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "Pause interval: " + PauseInMilliseconds.ToString());


                String _MaximumRefreshedWebs = ConfigurationManager.AppSettings["MaximumRefreshedWebs"];
				iMaximumRefreshedWebs = Int32.Parse(_MaximumRefreshedWebs);
				if((iMaximumRefreshedWebs < 0) || (iMaximumRefreshedWebs > 100))
				{
					iMaximumRefreshedWebs = 100;
				}
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "MaximumRefreshedWebs has been set to: " + iMaximumRefreshedWebs.ToString());


				//KeepAliveLogActivity
                String _shouldLog = ConfigurationManager.AppSettings["KeepAliveLogActivity"];
				if(_shouldLog == null)
					_shouldLog ="no";

				KeepAliveLogActivity= (_shouldLog.ToLower() == "yes")?true:false;
			}
			catch(Exception ex)
			{
				if(KeepAliveLogActivity == true)
					EventLog.WriteEntry("HumanityTHUNDER", "Error (GetConfigurationSettings): " + ex.Message);
			}

		}

		/// <summary>
		/// Method: SetTimerInterval
		/// 2005,2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.06 by ken norwick
		/// This method will use the interval setting (in minutes) that is
		/// found in the configuration file to calculate an interval in
		/// milliseconds that can be used to set the system timer interval
		/// property. The setting in minutes is multiplied by 60000 to calculate
		/// the interval for the timer. If the setting value is out of range or 
		/// missing, then this function will default to 4 minute intervals.
		/// </summary>
		/// <returns></returns>
		private int SetTimerInterval()
		{
			int _interval;
			String KeepAliveInterval;
			try
			{
                KeepAliveInterval = ConfigurationManager.AppSettings["KeepAliveInterval"];
			}
			catch
			{
				//catch missing or NULL value from XML config file
				//default to every 4 minutes
				KeepAliveInterval = "4";
			}
			int tmpInterval = Int32.Parse(KeepAliveInterval);
			if((tmpInterval < 1) || (tmpInterval > 99))
			{
				tmpInterval = 4;
			}
													
			_interval = tmpInterval * 60000;
			if(KeepAliveLogActivity == true)
				EventLog.WriteEntry("HumanityTHUNDER", "Service Timer Interval has been set to:" + tmpInterval.ToString() + " minutes.");
			return _interval;
		}


		/// <summary>
		/// EventHandler: OnStop()
		/// 2005, 2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.06 by ken norwick
		/// This handler provides support for a service Stop event occurrence. The function
		/// turns the system timer off.
		/// </summary>
		protected override void OnStop()
		{
			// TODO: Add code here to perform any tear-down necessary to stop your service.
			HumanityTimer.Enabled = false;
		}

		/// <summary>
		/// EventHandler: OnPause()
		/// 2005, 2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.06 by ken norwick
		/// This handler processes the Pause event for the service controller. It disables
		/// the system timer, and the service waits for a re-start command event.
		/// </summary>
		protected override void OnPause()
		{
			HumanityTimer.Enabled = false;
		}

		/// <summary>
		/// EventHandler: OnContinue()
		/// 2005,2006 Advance Guard Technologies - All Rights reserved
		/// Created: 2005.05.06 by ken norwick
		/// This event handler processes a re-start command from the master service controller.
		/// It retrieves any new configuration settings, and applies them before processing its
		/// task list. It then resets the system timer and processing occurs normally.
		/// </summary>
		protected override void OnContinue()
		{
			try
			{
				GetConfigurationSettings();
				GetConfiguredWebs();

				HumanityTimer.AutoReset = true;
				HumanityTimer.Interval = SetTimerInterval();
				HumanityTimer.Enabled = true;
				HumanityTimer.Start();
			}
			catch
			{

			}
		}

	}
}
